--[[
	BuffAllReady.lua - Core module for the BuffAllReady addon
	Written: Naiad - Frostmourne; 28-Jan-08
]]

local options = {
	type = 'group',
	args = {
		bom = {
			type = 'range',
			name = 'bom',
			desc = 'Blessing of Might priority',
			get = 'GetPriority',
			set = 'SetPriority',
			passValue = 'BOM',
			min = 1,
			max = 6,
			step = 1
		},
		bok = {
			type = 'range',
			name = 'bok',
			desc = 'Blessing of Kings priority',
			get = 'GetPriority',
			set = 'SetPriority',
			passValue = 'BOK',
			min = 1,
			max = 6,
			step = 1
		},
		bos = {
			type = 'range',
			name = 'bos',
			desc = 'Blessing of Salvation priority',
			get = 'GetPriority',
			set = 'SetPriority',
			passValue = 'BOS',
			min = 1,
			max = 6,
			step = 1
		},
		bol = {
			type = 'range',
			name = 'bol',
			desc = 'Blessing of Light priority',
			get = 'GetPriority',
			set = 'SetPriority',
			passValue = 'BOL',
			min = 1,
			max = 6,
			step = 1
		},
		bsc = {
			type = 'range',
			name = 'bsc',
			desc = 'Blessing of Sanctuary priority',
			get = 'GetPriority',
			set = 'SetPriority',
			passValue = 'BSC',
			min = 1,
			max = 6,
			step = 1
		},
		bow = {
			type = 'range',
			name = 'bow',
			desc = 'Blessing of Wisdom priority',
			get = 'GetPriority',
			set = 'SetPriority',
			passValue = 'BOW',
			min = 1,
			max = 6,
			step = 1
		}
	}
}

BuffAllReady = AceLibrary("AceAddon-2.0"):new( "AceConsole-2.0"
                                       , "AceEvent-2.0"
									   , "AceDB-2.0"
									   )
									   
BuffAllReady:RegisterChatCommand("/BuffAllReady", "/bar", options)

BuffAllReady:RegisterDB("BuffAllReadyDB", "BuffAllReadyDBPC")
BuffAllReady:RegisterDefaults("profile",
	{
		pallyBuffs = { 'BOM', 'BOK', 'BOS', 'BOL', 'BSC', 'BOW' }
	}
)

function BuffAllReady:OnInitialize()
	-- Called when the addon is loaded
end

function BuffAllReady:OnEnable()
    -- Called when the addon is enabled
	self:Print("Enabled")
	self:RegisterEvent("READY_CHECK")
end

function BuffAllReady:OnDisable()
    -- Called when the addon is disabled
	self:Print("Disabled")
end

function BuffAllReady:READY_CHECK()
	-- Called when someone does a ready check
	zone = GetZoneText();

	if zone ~= 'Stormwind City' and zone ~= 'City of Ironforge'
	and zone ~= 'Darnassus'	and zone ~= 'Undercity'
	and zone ~= 'Orgrimmar' and zone ~= 'Thunder Bluff'
	and zone ~= 'Shattrath City'
	then
		BuffAllReady:Announce()
	end
end

function BuffAllReady:Announce()
	-- Announce missing buffs to the raid
	self:Print("Announcing missing buffs..");
	
	numPriest  = 0
	numDruid   = 0
	numMage    = 0
	numPaladin = 0

	-- Determine player class
	lc, playerClass = UnitClass("player");
	
	-- Determine size of raid
	raidSize = GetNumRaidMembers();

	-- Determine chat channel to broadcast on
	if raidSize == 0
	then
		channel = PARTY

		-- Determine party buffer availability
		party = { "party1", "party2", "party3", "party4", "party5" }
		for i=1,5
		do
			lc, class = UnitClass(party[i]);
			if     class == "PRIEST"  then numPriest  = numPriest  + 1
			elseif class == "DRUID"   then numDruid   = numDruid   + 1
			elseif class == "MAGE"    then numMage    = numMage    + 1
			elseif class == "PALADIN" then numPaladin = numPaladin + 1
			end
		end
	else
		channel = RAID
		
		-- Determine raid buffer availability
		for i=1,raidSize
		do
			name, rank, subgroup, level, class, fileName, zone, online, isDead
			, role, isML = GetRaidRosterInfo(i);

			if     class == "Priest"  then numPriest  = numPriest  + 1
			elseif class == "Druid"   then numDruid   = numDruid   + 1
			elseif class == "Mage"    then numMage    = numMage    + 1
			elseif class == "Paladin" then numPaladin = numPaladin + 1
			end
		end
	end

	-- Need fort?
	if numPriest == 0 then needFort = 0 else	needFort = 1 end
	buffName, buffRank = GetPlayerBuffName("Power Word: Fortitude");
	if buffRank then needFort = 0 end
	buffName, buffRank = GetPlayerBuffName("Prayer of Fortitude");
	if buffRank then needFort = 0 end

	-- Need motw?
	if numDruid == 0 then needMotW = 0 else needMotW = 1 end
	buffName, buffRank = GetPlayerBuffName("Mark of the Wild");
	if buffRank then needMotW = 0 end
	buffName, buffRank = GetPlayerBuffName("Gift of the Wild");
	if buffRank then needMotW = 0 end

	-- Need ai?
	if numMage == 0 or playerClass == "ROGUE" or playerClass == "WARRIOR"
	then needAI = 0
	else needAI = 1
	end
	buffName, buffRank = GetPlayerBuffName("Arcane Intellect");
	if buffRank then needAI = 0 end
	buffName, buffRank = GetPlayerBuffName("Arcane Brilliance");
	if buffRank then needAI = 0 end

	-- Pally buffs
	needBOM = 0
	needBOK = 0
	needBOS = 0
	needBSC = 0
	needBOL = 0
	needBOW = 0
	for i=1,numPaladin
	do
		if self.db.profile.pallyBuffs[i] == 'BOM' then needBOM = 1 end
		if self.db.profile.pallyBuffs[i] == 'BOK' then needBOK = 1 end
		if self.db.profile.pallyBuffs[i] == 'BOS' then needBOS = 1 end
		if self.db.profile.pallyBuffs[i] == 'BOL' then needBOL = 1 end
		if self.db.profile.pallyBuffs[i] == 'BSC' then needBSC = 1 end
		if self.db.profile.pallyBuffs[i] == 'BOW' then needBOW = 1 end
	end
	buffName, buffRank = GetPlayerBuffName("Blessing of Might");
	if buffRank then needBOM = 0 end
	buffName, buffRank = GetPlayerBuffName("Greater Blessing of Might");
	if buffRank then needBOM = 0 end
	buffName, buffRank = GetPlayerBuffName("Blessing of Kings");
	if buffRank then needBOK = 0 end
	buffName, buffRank = GetPlayerBuffName("Greater Blessing of Kings");
	if buffRank then needBOK = 0 end
	buffName, buffRank = GetPlayerBuffName("Blessing of Salvation");
	if buffRank then needBOS = 0 end
	buffName, buffRank = GetPlayerBuffName("Greater Blessing of Salvation");
	if buffRank then needBOS = 0 end
	buffName, buffRank = GetPlayerBuffName("Blessing of Light");
	if buffRank then needBOL = 0 end
	buffName, buffRank = GetPlayerBuffName("Greater Blessing of Light");
	if buffRank then needBOL = 0 end
	buffName, buffRank = GetPlayerBuffName("Blessing of Sanctuary");
	if buffRank then needBSC = 0 end
	buffName, buffRank = GetPlayerBuffName("Greater Blessing of Sanctuary");
	if buffRank then needBSC = 0 end
	buffName, buffRank = GetPlayerBuffName("Blessing of Wisdom");
	if buffRank then needBOW = 0 end
	buffName, buffRank = GetPlayerBuffName("Greater Blessing of Wisdom");
	if buffRank then needBOW = 0 end

	-- presentation woot
	bunny = { '! ()()    '
	        , '! (*-*)   '
			, '! c(")(") '
			, '!         '
			, '!         '
			, '!         '
			, '!         '
			, '!         '
			, '!         ' }
	bunnyLine = 1

	if needFort == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Power Word: Fortitude"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needMotW == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Mark of the Wild"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needAI == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Arcane Intellect"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needBOM == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Blessing of Might"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needBOK == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Blessing of Kings"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needBOS == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Blessing of Salvation"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needBOL == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Blessing of Light"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needBSC == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Blessing of Sanctuary"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
	if needBOW == 1
	then
		SendChatMessage( bunny[bunnyLine] .. "Missing! Blessing of Wisdom"
		               , channel )
		bunnyLine = bunnyLine + 1
	end
			
	self:Print("Done")
end

function BuffAllReady:GetPriority(buff)
	-- look up the priority of a pally buff in the array
	for ap, ab in pairs(self.db.profile.pallyBuffs)
	do
		if ab == buff then return ap end
	end
	return 0
end

function BuffAllReady:SetPriority(newBuff, newPriority)
	-- swap the pally buff with the old buff at the new priority
	oldBuff = self.db.profile.pallyBuffs[newPriority]
	oldPriority = BuffAllReady:GetPriority(newBuff)

	self.db.profile.pallyBuffs[oldPriority] = oldBuff
	self.db.profile.pallyBuffs[newPriority] = newBuff
end

